package example;

import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.EValidator;
import org.eclipse.emf.ecore.util.Diagnostician;
import org.eclipse.ocl.ParserException;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.OCL;
import org.eclipse.ocl.examples.pivot.delegate.OCLDelegateDomain;
import org.eclipse.ocl.examples.pivot.delegate.OCLInvocationDelegateFactory;
import org.eclipse.ocl.examples.pivot.delegate.OCLSettingDelegateFactory;
import org.eclipse.ocl.examples.pivot.delegate.OCLValidationDelegateFactory;
import org.eclipse.ocl.examples.pivot.model.OCLstdlib;
import org.eclipse.ocl.examples.xtext.oclinecore.OCLinEcoreStandaloneSetup;
import org.eclipse.ocl.expressions.OCLExpression;
import org.eclipse.ocl.helper.OCLHelper;

import simplemodel.Person;
import simplemodel.SimplemodelFactory;
import simplemodel.SimplemodelPackage;

public class Example {

	public static void main(String[] args) {

		Example e = new Example();
		
		Person p = e.createPerson();
		
//		e.validate(p);
		e.validateBeforeSet(p);
	}
	
	private void validate(Person p) {
		// register Pivot globally (resourceSet == null)
		org.eclipse.ocl.examples.pivot.OCL.initialize(null);

		String oclDelegateURI = OCLDelegateDomain.OCL_DELEGATE_URI_PIVOT;
		EOperation.Internal.InvocationDelegate.Factory.Registry.INSTANCE.put(oclDelegateURI,
		    new OCLInvocationDelegateFactory.Global());
		EStructuralFeature.Internal.SettingDelegate.Factory.Registry.INSTANCE.put(oclDelegateURI,
		    new OCLSettingDelegateFactory.Global());
		EValidator.ValidationDelegate.Registry.INSTANCE.put(oclDelegateURI,
		    new OCLValidationDelegateFactory.Global());

		OCLinEcoreStandaloneSetup.doSetup();
		// install the OCL standard library
		OCLstdlib.install();
		
		Diagnostic diag = Diagnostician.INSTANCE.validate(p);
		
		System.out.println(diag);
	}
	
	private void validateBeforeSet(Person p) {

		OCL ocl = OCL.newInstance();
		OCLHelper helper = ocl.createOCLHelper();
		helper.setContext(SimplemodelPackage.Literals.PERSON);
		
		
		Constraint x = null;
		try {
			x = (Constraint) helper.createInvariant("father.age > age");
		} catch (ParserException e) {
			e.printStackTrace();
		}
		
		Object r = ocl.check(p, x);
		System.out.println(r);
	}

	private Person createPerson() {
		
		Person d = SimplemodelFactory.eINSTANCE.createPerson();
		d.setName("dad");
		d.setAge(50);
		
		Person s = SimplemodelFactory.eINSTANCE.createPerson();
		s.setName("Junior");
		s.setAge(10);
		s.setFather(d);
		
		return d;
	}

}
